%
% MATLAB_EM_Plot_3PH_Speed_Torque.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Ben Rowland 12 August 2019
%
% Plot the Three Phase Star Torque v Speed characteristics
%
% Note:
% use pathtool or Set Path
% to point to your working directory and include the ~/Primitives directory

clear all;

% include the ~/Primitives directory
addpath("Primitives");

% This forces the COM port to be closed, otherwise it can become locked
% Requires the Instrument control toolkit to allow this to function
delete(instrfindall)
clear EM_USB;
instrreset;

% Global variable holding the reference to the COM port, used by our
% Primitives library
global EM_USB;

% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
EM_USB = serial('COM5','Baudrate',115200,'Databits',8);
fopen(EM_USB);

% Early versions of the EM hardware may require EM_VERSION to be 0
% if comms is not working then try changing to 0
global EM_VERSION;
EM_VERSION = 1;

% This just tests the comms to the control unit before we start
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

% Preconditions
SetActive(0);         % Motor OFF
SetCapacitorBank(0);  % OFF
SetDynoLoad(127); % Max Resistance = Minimum Load
SetMotorDirection(0); % Clockwise
SetMotorType (2);     % Three Phase Motor
ZeroWeight();
SetACGenMode (1);     % Double Hump 3PH Accurate
SetACDriveMode (0);   % Analoge wave drive mode

frequency = 50;       % 50Hz
%motor_voltage = 10;   % 10V DC
dyno_load = 127;      % Dynamometer Load Resistance = 127 Ohms
SetFrequency(frequency);
pause(0.1)

% This just tests the comms to the control unit before we start
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

% Check the Dynamometer 'motor located' switches
switch_state = GetSwitches ();
if switch_state >= 1
   disp('==== AC MOTOR TEST START ====')
   disp('Motor Installed Correctly');
   current_I_U = GetCurrent(2);
   disp(['AC Current with motor OFF = ' , num2str(current_I_U) , 'A']);
   pause(0.1);
   %disp(['Setting Motor Voltage to ' , num2str(motor_voltage) , ' Volts']);
   %SetVoltage (0, motor_voltage);
   pause(0.1);
   disp(['Setting Dyno Load to ' , num2str(dyno_load) , ' Ohms']);
   SetDynoLoad(dyno_load);
   pause(0.1);
   current_I1 = GetCurrent(6);
   disp(['I1 Current (with Motor OFF) = ' , num2str(current_I1) , 'A']);    
   loop_count = 1000;
   disp('MOTOR ON');
   index = 1;
   SetActive(1); % Motor ON
   pause(0.1); % Wait for the motor to spin up

   % Perform the sampled measurements in as tight a loop as you can.
   for loop = 1:127
      load_cell_torque_range(index) = GetLoadCell(1);
      encoder_speed_range(index) = GetRPM();
      disp(['Dyno Load = ' , num2str(dyno_load) , 'R - RPM = ' , num2str(encoder_speed_range(index)) , ' - Torque = ' , num2str(load_cell_torque_range(index))]); 

      dyno_load = 127 - loop; 
      SetDynoLoad(loop);
      
      index = index + 1;
      pause(0.5);
   end

   % Sampling over, you can turn off the motor
   % and close the COM port
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(EM_USB)
  
   % Plot the RPM against Time
   curent_torque_plot = figure;
   figure(curent_torque_plot);
   movegui(curent_torque_plot , 'northwest');
   hold on
   p2 = polyfit(encoder_speed_range, load_cell_torque_range, 3); % best fit curve
   y2 = polyval(p2, encoder_speed_range);
   plot(encoder_speed_range, load_cell_torque_range, 'ro', encoder_speed_range, y2, '-r' );
   %axis(0, 2000, -5000, inf);
   xlim([0 1600]); % 0 to 1500 RPM
   %ylim([nn mm]); TO DO
   torque_speed_polynomial = poly2sym(p2,sym('RPM'));
   torque_speed_equation = sprintf('Torque = %s',torque_speed_polynomial); 
   disp(torque_speed_equation);
   grid on
   legend('Samples','Best fit curve','Location','northeast');
   xlabel('Speed, RPM');
   ylabel('Torque, mNm');
   title(['Three Phase Star Motor Speed v Torque@ ', num2str(frequency) , 'Hz']);      
   hold off
   
   pause(0.1);
   disp('==== AC MOTOR TEST END ======');
else    
   disp('==== Motor not fitted in Dynomometer ====');
end

% Check for corrupt comms
fopen(EM_USB);
api_version_two = GetAPIVersion();
if api_version == api_version_two
   % do nothing
else
    disp('=== CORRUPT COMMS ===');
    disp(['api_version = ' , num2str(api_version_two) , ' it was ' , num2str(api_version) ] );
end
   
fclose(EM_USB) % Close the COM port

% MATLAB_EM_Plot_DC_PM_Torque_Speed.m

