%
% MATLAB_EM_Plot_DC_PM_Speed_Torque.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Ben Rowland 12 August 2019
%
% Plot the DC Permanent Magnet Motor Speed v Torque characteristics
%

clear all;

% include the ~/Primitives directory
addpath("Primitives");

% This forces the COM port to be closed, otherwise it can become locked
% Requires the Instrument control toolkit to allow this to function
delete(instrfindall)
clear EM_USB;
instrreset;

% Global variable holding the reference to the COM port, used by our
% Primitives library
global EM_USB;

% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
EM_USB = serial('COM13','Baudrate',115200,'Databits',8);
fopen(EM_USB);

% Early versions of the EM hardware may require EM_VERSION to be 0
% if comms is not working then try changing to 0
global EM_VERSION;
EM_VERSION = 1;

% This just tests the comms to the control unit before we start
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

SetActive(0);         % Motor OFF
SetCapacitorBank(0);  % OFF
SetDynoLoad(127); % Max Resistance = Minimum Load
SetMotorDirection(0); % Clockwise
SetMotorType (0);     % DC Motor
ZeroWeight();

motor_voltage = 20;   % 20V DC
dyno_load = 127;      % Dynamometer Load Resistance = 127 Ohms

% This just tests the comms to the control unit before we start
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

% Check the Dynamometer 'motor located' switches
switch_state = GetSwitches ();
if switch_state >= 1
   disp('==== DC MOTOR TEST START ====')
   disp('Motor Installed Correctly');
   current_I_DC = GetCurrent(0);
   disp(['DC Current with motor OFF = ' , num2str(current_I_DC) , 'A']);

   disp(['Setting Motor Voltage to ' , num2str(motor_voltage) , ' Volts']);
   SetVoltage (0, motor_voltage);

   disp(['Setting Dyno Load to ' , num2str(dyno_load) , ' Ohms']);
   SetDynoLoad(0);

   current_I1 = GetCurrent(6);
   disp(['I1 Current (with Motor OFF) = ' , num2str(current_I1) , 'A']);    
   loop_count = 1000;
   disp('MOTOR ON');
   
   index = 1;
   SetActive(1); % Motor ON
   pause(2.0); % Wait for the motor to spin up

   % Perform the sampled measurements in as tight a loop as you can.
   for loop = 1:127
      load_cell_torque_range(index) = GetLoadCell(1);
      encoder_speed_range(index) = GetRPM();
      disp(['Dyno Load = ' , num2str(dyno_load) , 'R - RPM = ' , num2str(encoder_speed_range(index)) , ' - Torque = ' , num2str(load_cell_torque_range(index))]);    
    
      dyno_load = 127 - loop; 
      SetDynoLoad(loop);
     
      index = index + 1;
      pause(0.5);
   end

   % Sampling over, you can turn off the motor
   % and close the COM port
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(EM_USB);
  
   % Plot the RPM against Time
   curent_torque_plot = figure;
   figure(curent_torque_plot);
   movegui(curent_torque_plot , 'northwest');
   hold on
   p2 = polyfit(encoder_speed_range, load_cell_torque_range , 1); % line of best fit
   y2 = polyval(p2, encoder_speed_range);
   plot(encoder_speed_range, load_cell_torque_range, 'ro', encoder_speed_range, y2, '-r' );
   current_speed_polynomial = poly2sym(p2,sym('T'));
   current_speed_equation = sprintf('Speed = %s',current_speed_polynomial); 
   disp(current_speed_equation);
   grid on
   legend('Samples','Best fit line','Location','southeast');
   xlabel('Speed, RPM');
   ylabel('Torque, mNm');
   title(['PM DC Motor Speed v Torque @ ', num2str(motor_voltage) , 'V']);      
   hold off
   
   pause(0.1);
   disp('==== DC MOTOR TEST END ======');
else    
   disp('==== Motor not fitted in Dynomometer ====');
end

% Check for corrupt comms
fopen(EM_USB);
api_version_two = GetAPIVersion();
if api_version == api_version_two
   % do nothing
else
    disp('=== CORRUPT COMMS ===');
    disp(['api_version = ' , num2str(api_version_two) , ' it was ' , num2str(api_version) ] );
end
   
fclose(EM_USB); % Close the COM port

% MATLAB_EM_Plot_DC_PM_Speed_Torque.m


